/**
 * Alex Parkhurst, Nathan Lambert, Brian Gross
 * anp56 - nol5 - bwg38
 *
 * ECE 4760 Final Project
 * Resistor Sorter
 */


// adapted from https://people.ece.cornell.edu/land/courses/ece4760/PIC32/ProtoThreads/Schedule_uart_switch.c
////////////////////////////////////
// clock AND protoThreads configure!
// You MUST check this file!
#include "config.h"
// threading library
#include "pt_cornell_1_2_1.h"
#include "tft_master.h"
#include "tft_gfx.h"
#include <stdlib.h>
//#include <stdio.h> already in config.h

#define MAIN_MENU_OPTION_LIM 2
#define SORTING_MENU_OPTION_LIM 2
#define PWM_GO_UP 2200
#define PWM_GO_DOWN 5000

// === thread structures ============================================

// note that UART input and output are threads
static struct pt pt_control, pt_print, pt_rotate ;

//period in 64 prescaled cycles for the PWM period and PWM on time.
char buffer[128];

//period in 128 prescaled cycles for the PWM period and PWM on time.
int PWM_period=20000, pwm_on_time=5000; //5000 turn right //2200 turn left clock

////////////////////////////////////////////////////////////////////////////////
//NEW THINGS BELOW
// state and display variables
volatile char state= 't';
volatile int opt= 0;

//vector to hold bin limits
int bin_lim_disp[6]= {0,0,0,0,0,0};
//bin_lim is the lower limit of each bin's range
int bin_lim[6]= {0,0,0,0,0,0};
char bin_lim_exp[6]= {' ',' ',' ',' ',' ',' '};
char exp_chars[3]= {' ','k','M'}; 

int custom_opt= 1;
int custom_b= 0;
int exponentiation[4]= {1, 1000, 1000000};

int raw_adc ;
float R;
float I[4]={550e-6, 0.55e-6, 5.5e-6, 55e-6} ; // current settings in amps
int adc_zero[4]={124, 0, 0, 22}; // ADC reading with zero ohms at each current
//need to fix 1 and 2, 2 in particular seems to have a lower current
int i;
int current_option;
int cur_bin =0;
int command_bin =0;

//input vars
static int adc_5;
static float scaled_adc;
static char b_state= 'r';
static int adc_5_prev= 0;
static int tButton;//for keeping track of the button press
static int uButton; //input variable for button
//selection vars
static char menu_opt[MAIN_MENU_OPTION_LIM]= {'m', 'o'};
static int sorting_opt[SORTING_MENU_OPTION_LIM]= {'s', 'c'}; 

//STATE VARIABLES:
// t - start menu state
// m - measure state
// o - choose sorting state
// g - logarithmic preset
// i - linear preset
// c - custom sorting
// s - sorting state

// Print helper function: to print the entire line in selecting custom bin limits
void mag_select(int select_var, int y_posn){
switch(select_var) {
    case 0: // Ohm selected
        // ohm
        tft_fillRoundRect(195, y_posn, 45, 20, 1, ILI9340_WHITE);
        tft_setTextColor(ILI9340_BLACK);
        tft_setCursor(200, y_posn);
        sprintf(buffer, "Ohm");                                
        tft_writeString(buffer);
        // kilo
        tft_fillRoundRect(245, y_posn, 20, 20, 1, ILI9340_BLACK);
        tft_setTextColor(ILI9340_WHITE);                                
        tft_setCursor(250, y_posn);
        sprintf(buffer, "k");
        tft_writeString(buffer);
        // mega
        tft_fillRoundRect(270, y_posn, 20, 20, 1, ILI9340_BLACK);
        tft_setTextColor(ILI9340_WHITE);                                
        tft_setCursor(275, y_posn);
        sprintf(buffer, "M");
        tft_writeString(buffer);

        break;
    case 1://kOhm selected
        // Ohm
        tft_fillRoundRect(195, y_posn, 45, 20, 1, ILI9340_BLACK);
        tft_setTextColor(ILI9340_WHITE);
        tft_setCursor(200, y_posn);
        sprintf(buffer, "Ohm");                                
        tft_writeString(buffer);
        //k
        tft_fillRoundRect(245, y_posn, 20, 20, 1, ILI9340_WHITE);
        tft_setTextColor(ILI9340_BLACK);                                
        tft_setCursor(250, y_posn);
        sprintf(buffer, "k");
        tft_writeString(buffer);
        //M
        tft_fillRoundRect(270, y_posn, 20, 20, 1, ILI9340_BLACK);
        tft_setTextColor(ILI9340_WHITE);                                
        tft_setCursor(275, y_posn);
        sprintf(buffer, "M");
        tft_writeString(buffer);
        break;
    case 2:// MOhm is selected
        //Ohm
        tft_fillRoundRect(195, y_posn, 45, 20, 1, ILI9340_BLACK);
        tft_setTextColor(ILI9340_WHITE);
        tft_setCursor(200, y_posn);
        sprintf(buffer, "Ohm");                                
        tft_writeString(buffer);
        //k
        tft_fillRoundRect(245, y_posn, 20, 20, 1, ILI9340_BLACK);
        tft_setTextColor(ILI9340_WHITE);                                
        tft_setCursor(250, y_posn);
        sprintf(buffer, "k");
        tft_writeString(buffer);
        //M
        tft_fillRoundRect(270, y_posn, 20, 20, 1, ILI9340_WHITE);
        tft_setTextColor(ILI9340_BLACK);                                
        tft_setCursor(275, y_posn);
        sprintf(buffer, "M");
        tft_writeString(buffer);
        break;
}
}

//Print Helper Function: Selects Measure vs. Sort States.
void start_mode_select(int opt){
    switch (opt){
    case 0:// Measure selected
        tft_setTextSize(2);
        tft_fillRoundRect(35, 60, 115, 35, 1, ILI9340_WHITE);
        tft_setCursor(40, 70);
        tft_setTextColor(ILI9340_BLACK);
        sprintf(buffer,"Measure");
        tft_writeString(buffer);
        tft_fillRoundRect(35, 100, 115, 35, 1, ILI9340_BLACK);
        tft_setCursor(40, 110);
        tft_setTextColor(ILI9340_WHITE);
        sprintf(buffer,"Sort");
        tft_writeString(buffer);
        break;
    case 1:// Sort is selected
        tft_setTextSize(2);
        tft_fillRoundRect(35, 60, 115, 35, 1, ILI9340_BLACK);
        tft_setCursor(40, 70); 
        tft_setTextColor(ILI9340_WHITE);
        sprintf(buffer,"Measure");
        tft_writeString(buffer);
        tft_fillRoundRect(35, 100, 115, 35, 1, ILI9340_WHITE);
        tft_setCursor(40, 110);
        tft_setTextColor(ILI9340_BLACK);
        sprintf(buffer,"Sort");
        tft_writeString(buffer);
        break;
}
}

// Print Helper function -- Selects Logrithmic vs Custom Sort
void which_sort_select(int opt){
    switch (opt){
    case 0:
        //Logarithmic is selected
        tft_setTextSize(2);
        tft_fillRoundRect(35, 80, 150, 35, 1, ILI9340_WHITE);
        tft_setTextColor(ILI9340_BLACK);
        tft_setCursor(40, 90);
        sprintf(buffer,"Logarithmic");
        tft_writeString(buffer);
        tft_fillRoundRect(35, 120, 150, 35, 1, ILI9340_BLACK);
        tft_setTextColor(ILI9340_WHITE);
        tft_setCursor(40, 130);
        sprintf(buffer,"Custom");
        tft_writeString(buffer);
        tft_setTextSize(2);
        tft_fillRoundRect(5, 190, 320, 50, 1, ILI9340_BLACK);
        tft_setCursor(10, 200);
        sprintf(buffer,"0-100, 102-1k, 1k-10k, ");
        tft_writeString(buffer);
        tft_setCursor(10, 220);
        sprintf(buffer,"10k-100k, 100k-inf");
        tft_writeString(buffer);
        break;
    case 1:
        //Custom is selected
        tft_setTextSize(2);
        tft_fillRoundRect(35, 80, 150, 35, 1, ILI9340_BLACK);
        tft_setTextColor(ILI9340_WHITE);
        tft_setCursor(40, 90);
        sprintf(buffer,"Logarithmic");
        tft_writeString(buffer);
        tft_fillRoundRect(35, 120, 150, 35, 1, ILI9340_WHITE);
        tft_setTextColor(ILI9340_BLACK);
        tft_setCursor(40, 130);
        sprintf(buffer,"Custom");
        tft_writeString(buffer);
        tft_fillRoundRect(5, 190, 320, 50, 1, ILI9340_BLACK);
}
}

// === DISPLAY THREAD ======================================================
// Display thread: Updates every 1/5 of a second the TFT display
static PT_THREAD (protothread_print(struct pt *pt)){
PT_BEGIN(pt);
tft_fillScreen(ILI9340_BLACK);
while(1) {
    switch (state){
        case 't':
            //start menu state
            tft_setTextColor(ILI9340_WHITE);  tft_setTextSize(4);
            tft_setCursor(20, 20);
            sprintf(buffer,"Start Menu");
            tft_writeString(buffer);
            start_mode_select(opt);
            break;

        case 'o':
            //choose log bins vs custom bins
            tft_setTextColor(ILI9340_WHITE);  tft_setTextSize(3);
            tft_setCursor(20, 20);
            sprintf(buffer,"Choose Sorting");
            tft_writeString(buffer);
            tft_setCursor(20, 55); 
            sprintf(buffer,"Preset");
            tft_writeString(buffer);
            which_sort_select(opt);
            break; 
        case 'c':
            //custom bin selection
            tft_setTextColor(ILI9340_WHITE);
            tft_setTextSize(4);
            tft_setCursor(10, 20);
            sprintf(buffer, "Custom Bin Limit");
            tft_writeString(buffer);

            switch (custom_opt){
                case 1:
                    tft_setTextSize(2);
                    tft_setCursor(10, 60);
                    tft_fillRoundRect(10, 60, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin1:0-%d%c", bin_lim_disp[1], bin_lim_exp[1]);
                    tft_writeString(buffer);
                    mag_select(opt, 60);
                    break;
                case 2:
                    tft_setTextSize(2);
                    tft_setCursor(10, 60);
                    tft_fillRoundRect(10, 60, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin1:0-%d%c", bin_lim_disp[1], bin_lim_exp[1]);
                    tft_writeString(buffer);
                    tft_setCursor(10, 90);
                    tft_fillRoundRect(10, 90, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin2:%d%c-%d%c", bin_lim_disp[1], bin_lim_exp[1], bin_lim_disp[2], bin_lim_exp[2]);
                    tft_writeString(buffer);
                    mag_select(opt, 90);
                    break;
                case 3:
                    tft_setTextSize(2);
                    tft_setCursor(10, 90);
                    tft_fillRoundRect(10, 90, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin2:%d%c-%d%c", bin_lim_disp[1], bin_lim_exp[1], bin_lim_disp[2], bin_lim_exp[2]);
                    tft_writeString(buffer);
                    tft_setCursor(10, 120);
                    tft_fillRoundRect(10, 120, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin3:%d%c-%d%c", bin_lim_disp[2], bin_lim_exp[2], bin_lim_disp[3], bin_lim_exp[3]);
                    tft_writeString(buffer);
                    mag_select(opt, 120);
                    break;
                case 4:
                    tft_setTextSize(2);
                    tft_setCursor(10, 120);
                    tft_fillRoundRect(10, 120, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin3:%d%c-%d%c", bin_lim_disp[2], bin_lim_exp[2], bin_lim_disp[3], bin_lim_exp[3]);
                    tft_writeString(buffer);
                    tft_setCursor(10, 150);
                    tft_fillRoundRect(10, 150, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin4:%d%c-%d%c", bin_lim_disp[3], bin_lim_exp[3], bin_lim_disp[4], bin_lim_exp[4]);
                    tft_writeString(buffer);                
                    mag_select(opt, 150);
                    break;
                case 5:
                    tft_setTextSize(2);
                    tft_setCursor(10, 150);
                    tft_fillRoundRect(10, 150, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin4:%d%c-%d%c", bin_lim_disp[3], bin_lim_exp[3], bin_lim_disp[4], bin_lim_exp[4]);
                    tft_writeString(buffer);                
                    tft_setCursor(10, 180);
                    tft_fillRoundRect(10, 180, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin5:%d%c-%d%c", bin_lim_disp[4], bin_lim_exp[4], bin_lim_disp[5], bin_lim_exp[5]);
                    tft_writeString(buffer);
                    mag_select(opt, 180);
                    break;
                case 6:
                    tft_setTextSize(2);
                    tft_setCursor(10, 180);
                    tft_fillRoundRect(10, 180, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin5:%d%c-%d%c", bin_lim_disp[4], bin_lim_exp[4], bin_lim_disp[5], bin_lim_exp[5]);
                    tft_writeString(buffer);
                    tft_setCursor(10, 210);
                    tft_fillRoundRect(10, 210, 175, 20, 1, ILI9340_BLACK);
                    sprintf(buffer, "Bin6:%d%c-inf", bin_lim_disp[5], bin_lim_exp[5]);
                    tft_writeString(buffer);
                    mag_select(opt, 210);
                    break;
        }
            break;
        default:
            //in measure and sort threads printing does not occur here.
            break;
    }
// print every 200 mSec
PT_YIELD_TIME_msec(200) ;
} // END WHILE(1)
  PT_END(pt);
}

//==================== ROTATE PROTOTHREAD ===============
// This protothread is spawned by the control protothread to rotate the stepper
// motor to the commanded bin. This thread takes in cur_bin, the current bin
// underneath the drop point, and command_bin, the desired bin to drop into. It
// then rotates the stepper so that the commanded bin is under the drop point
static PT_THREAD (protothread_rotate(struct pt *pt)){
PT_BEGIN(pt);
    mPORTBSetPinsDigitalOut(BIT_8|BIT_9|BIT_10|BIT_5);//Make port B3 output
    mPORTBClearBits(BIT_8|BIT_9|BIT_10|BIT_5);//Clear Output
    //Stepper Bits go: 8-Brown, 9-Black, 10-Yellow, 5-Orange.
    static int num_turns;
    static int t;
    num_turns = 2*((command_bin - cur_bin +6)%6);
    //number of turns we need to make 2 turns per bin
    for(t=0; t<num_turns; t++) {//now loop through and drive the stepper motor
        mPORTBClearBits(BIT_9|BIT_5);
        mPORTBSetBits(BIT_8|BIT_10);
        PT_YIELD_TIME_msec(100);
        mPORTBClearBits(BIT_8);
        mPORTBSetBits(BIT_9);
        PT_YIELD_TIME_msec(100);
        mPORTBClearBits(BIT_10);
        mPORTBSetBits(BIT_5);
        PT_YIELD_TIME_msec(100);
        mPORTBClearBits(BIT_9);
        mPORTBSetBits(BIT_8);
        PT_YIELD_TIME_msec(100);
        if (cur_bin == 5) {
            cur_bin = 0;
        }
        else {
            cur_bin++;
        }
    }
    mPORTBClearBits(BIT_8|BIT_5); //turn off so stepper doesn't draw idle power
PT_EXIT(pt);
PT_END(pt);
} 

// Debounce function: returns 0 if not debounced yet, 1 if ready to continue. 
int debounce() {
    switch (b_state) {
        case 'r': //RELEASE
            if (uButton){
                tButton= uButton;
                b_state= 'm';
            }
            break;
        case 'm': //MAYBE PUSHED
            if (uButton == tButton) {
                b_state = 'p'; //to maybe released
            } 
            break;
        case 'p': // STILL PUSHED
            if (uButton != tButton){
                b_state = 'n'; //change state to MAYBE RELEASED
            }
            break;
        case 'n': //MAYBE RELEASED
            if (uButton == tButton) {
                b_state = 'p'; //change state to STILL PUSHED
            }
            else {
                b_state = 'r';
                return 1;
            }   
        }
    return 0;
}

//helper function to determine the appropriate bin to sort into
void determine_bin(int res) {
    for(i = 1; i<6; i++) {
        if (bin_lim[i] > res) {
            command_bin = i;
            return;
        }
    }
    command_bin =0;
}

//helper function to calculate the resistance based on the current mode
//and the regression-derived parameters.
void calculate_resistance() {
    if (current_option == 0) {
        R = 5.9714*((float) raw_adc) -758.8;
    }
    else if (current_option == 3) {
        R = 61.661*((float) raw_adc) - 959.92;
    }
    else if (current_option == 2) {
        R = 618.12*((float) raw_adc) - 2679.1;
    }
    else { // == 1
        R = (0.0848*((float) raw_adc)*((float) raw_adc)*((float) raw_adc) - 
                44.692*((float) raw_adc)*((float) raw_adc)+
                14147.0*((float) raw_adc) - 518275.0);
    }
}

// === CONTROL THREAD ======================================================
// This thread is the main control thread for the resistor sorting 
static PT_THREAD (protothread_control(struct pt *pt)){
PT_BEGIN(pt);
while (1){
adc_5_prev= adc_5; //keep last pot reading
AcquireADC10(); // THOU SHALT NOT TURN ON ADC AUTOSAMPLE!
adc_5 = ReadADC10(0);   // read the result of channel 9 conversion from the idle buffer
uButton= mPORTAReadBits(BIT_4); //read for universal button every time -- universal wired into pin 7

switch(state){
    case 't':
        //start menu state
        //increment menu item
        if ((adc_5 - adc_5_prev) > 4){
            if (opt != (MAIN_MENU_OPTION_LIM - 1)){
                opt= opt + 1;
            }
        }
        else if ((adc_5 - adc_5_prev) < -4){
            if (opt != 0){
                opt= opt - 1;
            }
        }
        if (debounce() == 1) {
            tft_fillScreen(ILI9340_BLACK);
            //uButton= 0;
            state= menu_opt[opt];
        }
        PT_YIELD_TIME_msec(10) ;
        break;
    case 'm':
        //measure state
        tft_setTextColor(ILI9340_WHITE);
        tft_setTextSize(3);
        tft_setCursor(20, 20);
        sprintf(buffer, "Insert Resistor");
        tft_writeString(buffer);        
        tft_setCursor(20, 55);
        sprintf(buffer, "and Press Select");
        tft_writeString(buffer);
        if (debounce() == 1) {
            CTMUCONbits.ON = 1; // Turn on CTMU
            CTMUCONbits.IDISSEN = 0; // End drain of circuit
            CTMUCONbits.EDG1STAT = 1; // Begin charging the circuit
            //Send Servo up to measure the resistor
            SetDCOC1PWM(PWM_GO_UP); 
            PT_YIELD_TIME_msec(1000);
            SetDCOC1PWM(0);
            SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN0 ); // configure to sample Resistor
            for (i=0;i<4;i++){
                //step through each current level, starting at the lowest
                if (i == 3) {
                    current_option = 0;
                }
                else {
                    current_option = i+1;
                }
                CTMUCONbits.IRNG = current_option;
                CTMUCONbits.EDG1STAT = 1;
                PT_YIELD_TIME_msec(100);  //MUST have a Yield Time here, else ADC read doesn't work
                AcquireADC10(); // start ADC sampling
                while (!AD1CON1bits.DONE){}; // Wait for ADC conversion
                raw_adc =  ReadADC10(0) ;
                if (raw_adc > 99) { //if the ADC reading is high enough, this is a good current level
                   break; 
                }
            }
            calculate_resistance();
            command_bin =0; 
            tft_fillRoundRect(0,100, 320, 100, 1, ILI9340_BLACK);// x,y,w,h,radius,color
            tft_setCursor(0, 100);
            tft_setTextColor(ILI9340_WHITE); tft_setTextSize(2);
            sprintf(buffer,"Res: %6.1f Ohms", R );
            tft_writeString(buffer);
            tft_setCursor(0, 160);
            sprintf(buffer, "Raw:%d Adj:%d I:%d B:%d",raw_adc, (raw_adc- adc_zero[current_option]), current_option, command_bin );
            tft_writeString(buffer);
            
            PT_SPAWN(pt, &pt_rotate, protothread_rotate(&pt_rotate));
            SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN5 );
            SetDCOC1PWM(PWM_GO_DOWN); 
            PT_YIELD_TIME_msec(450);
            SetDCOC1PWM(0);
            PT_YIELD_TIME_msec(100);
            SetDCOC1PWM(PWM_GO_UP);
            PT_YIELD_TIME_msec(160);
            SetDCOC1PWM(0);
        }
        //measure state -- hard code output to rollback, display values
        //code resistor to return to user -- bin select servo direction
        //measure resistor helper function -- used repeatedly in sorting
        //display resistor value on screen -- update display 
        //wait for reset or future resistor, hold value on screen until either happens
        PT_YIELD_TIME_msec(10) ;

        break;
    case 'o':
        //control loop for option selector
        if ((adc_5 - adc_5_prev) > 4){
            //update menu, based on which is selected
            if (opt != (SORTING_MENU_OPTION_LIM - 1)){
                opt= opt + 1;
            }
        }
        else if ((adc_5 - adc_5_prev) < -4){
            if (opt != 0){
                opt= opt - 1;
            }
        }
        if (debounce() == 1) {
            tft_fillScreen(ILI9340_BLACK);
            //uButton= 0;
            state= sorting_opt[opt];
            if (state == 's') {
                bin_lim[1]= 100;
                bin_lim[2] = 1000;
                bin_lim[3] = 10000;
                bin_lim[4] = 100000;
                bin_lim[5] = 1000000000;
            }
        }
        PT_YIELD_TIME_msec(10) ;
        break;
    case 'c':
        //custom choose state
        scaled_adc = (float)adc_5 * 1000 / 958.0 ; // Vref*adc/1023    
        switch(custom_b){
            case 0:
                //control the value for the limits
                if ((int)scaled_adc < 20){
                    bin_lim_disp[custom_opt]= 1;
                }
                else if ((int)scaled_adc > 1000){
                    bin_lim_disp[custom_opt]= 1000;
                }
                else {
                    if ((adc_5 - adc_5_prev) > 1){
                        //update menu, based on which is selected
                        if (bin_lim_disp[custom_opt] < 20){
                            bin_lim_disp[custom_opt]= 20;
                        }
                        else if (bin_lim_disp[custom_opt] <= 980){
                            bin_lim_disp[custom_opt]= bin_lim_disp[custom_opt] + 20;
                        }
                    }
                    else if ((adc_5 - adc_5_prev) < -1){
                        if (bin_lim_disp[custom_opt] > 20){
                            bin_lim_disp[custom_opt]= bin_lim_disp[custom_opt] - 20;
                        } else {
                            bin_lim_disp[custom_opt]= 1;
                        }
                    }
                }
                break;
            case 1:
                //logic for choosing opt -- controls OOM
                if ((adc_5 - adc_5_prev) > 3){
                    //update menu, based on which is selected
                    if (opt != (2)){
                        opt= opt + 1;
                    }
                }
                else if ((adc_5 - adc_5_prev) < -3){
                    if (opt != 0){
                        opt= opt - 1;
                    }
                }
                break;
        }

        if (debounce() == 1) {
            if (custom_opt == 6){
                tft_fillScreen(ILI9340_BLACK);
                state= 's';
            }
            else if (custom_b == 0) {
                custom_b= 1;
            }
            else if (custom_b == 1){
                bin_lim_exp[custom_opt]= exp_chars[opt];
                bin_lim[custom_opt]= bin_lim_disp[custom_opt] * exponentiation[opt];
                custom_opt= custom_opt+1;
                custom_b= 0;
            }
        }
        PT_YIELD_TIME_msec(10) ;
        break;
    case 's':
        //Sort Mode
        tft_setTextColor(ILI9340_WHITE);  tft_setTextSize(3);
        tft_setCursor(20, 20);
        sprintf(buffer,"Sorting...");
        tft_writeString(buffer);
        //Charge time measurement setup
        CTMUCONbits.ON = 1; // Turn on CTMU
        CTMUCONbits.IDISSEN = 0; // End drain of circuit
        CTMUCONbits.EDG1STAT = 1; // Begin charging the circuit
        while(1) {
            mPORTBSetBits(BIT_7);//Turn on Solenoid
            PT_YIELD_TIME_msec(400);
            mPORTBClearBits(BIT_7);//Turn off Solenoid
            PT_YIELD_TIME_msec(5000);
            //Send Servo up to measure the resistor
            SetDCOC1PWM(PWM_GO_UP); 
            PT_YIELD_TIME_msec(500);
            SetDCOC1PWM(0);
            SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN0 ); // configure to sample Resistor
            for (i=0;i<4;i++){
                //step through each current level, starting at the lowest
                if (i == 3) {
                    current_option = 0;
                }
                else {
                    current_option = i+1;
                }
                CTMUCONbits.IRNG = current_option;
                CTMUCONbits.EDG1STAT = 1;
                PT_YIELD_TIME_msec(100);  //MUST have a Yield Time here, else ADC read doesn't work
                AcquireADC10(); // start ADC sampling
                while (!AD1CON1bits.DONE){}; // Wait for ADC conversion
                raw_adc =  ReadADC10(0) ;
                if (raw_adc > 99) { //if the ADC reading is high enough, this is a good current level
                   break; 
                }
            }
            calculate_resistance();
            determine_bin((int)R);
            tft_fillRoundRect(0,100, 320, 100, 1, ILI9340_BLACK);// x,y,w,h,radius,color
            tft_setCursor(0, 100);
            tft_setTextColor(ILI9340_WHITE); tft_setTextSize(2);
            //sprintf(buffer, '%d%d%d', bin_lim[0], bin_lim[1] 
            sprintf(buffer,"Res: %6.1f Ohms", R );
            tft_writeString(buffer);
            tft_setCursor(0, 160);
            sprintf(buffer, "Raw:%d Adj:%d I:%d B:%d",raw_adc, (raw_adc- adc_zero[current_option]), current_option, command_bin );
            tft_writeString(buffer);
            
            PT_SPAWN(pt, &pt_rotate, protothread_rotate(&pt_rotate));
            SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN5 ); // configure to sample Menus
            //Move Servo down to drop resistor, then bring it back up to recieve next one
            SetDCOC1PWM(PWM_GO_DOWN); 
            PT_YIELD_TIME_msec(450);
            SetDCOC1PWM(0);
            PT_YIELD_TIME_msec(100);
            SetDCOC1PWM(PWM_GO_UP);
            PT_YIELD_TIME_msec(160);
            SetDCOC1PWM(0);
            } // END WHILE(1)
        break;
}
}
PT_END(pt);
}

// === Main  ======================================================
// main sets up the pushbutton inputs, 
int main(void)
{
    // ====== ENSURE THAT AUXILLIARY FUNCTIONS OF THE CURRENT SOURCE ARE OFF
    ANSELA = 0; ANSELB = 0; CM1CON = 0; CM2CON = 0;

    // === config the uart, DMA, vref, timer5 ISR =============
    PT_setup();
    INTEnableSystemMultiVectoredInt();
  
    // =============== init I/O pins
    mPORTASetPinsDigitalIn(BIT_4|BIT_1); 
    mPORTBSetPinsDigitalOut(BIT_8|BIT_9|BIT_10|BIT_5|BIT_7);//Make port B3 output
    mPORTBClearBits(BIT_8|BIT_9|BIT_10|BIT_5|BIT_7);//Clear Output
    //bit 4 -- pin 12 -- is select button
    //bit 3 -- pin 10 -- is universal back button
  
    // ============init ADC =============================
    // configure and enable the ADC
    CloseADC10();	// ensure the ADC is off before setting the configuration
    // define setup parameters for OpenADC10
    // Turn module on | ouput in integer | trigger mode auto | enable autosample
    // ADC_CLK_AUTO -- Internal counter ends sampling and starts conversion (Auto convert)
    // ADC_AUTO_SAMPLING_ON -- Sampling begins immediately after last conversion completes; SAMP bit is automatically set
    // ADC_AUTO_SAMPLING_OFF -- Sampling begins with AcquireADC10();
    #define PARAM1  ADC_FORMAT_INTG16 | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_OFF //
    // define setup parameters for OpenADC10
    // ADC ref external  | disable offset test | disable scan mode | do 1 sample | use single buf | alternate mode off
    #define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF
    // Define setup parameters for OpenADC10
    // use peripherial bus clock | set sample time | set ADC clock divider
    // ADC_CONV_CLK_Tcy2 means divide CLK_PB by 2 (max speed)
    // ADC_SAMPLE_TIME_5 seems to work with a source resistance < 1kohm
    #define PARAM3 ADC_CONV_CLK_PB | ADC_SAMPLE_TIME_31 | ADC_CONV_CLK_32Tcy //ADC_SAMPLE_TIME_15| ADC_CONV_CLK_Tcy2
    // define setup parameters for OpenADC10
    // set AN4 and  as analog inputs
    #define PARAM4	ENABLE_AN0_ANA // pin 7 (AN5) for pot)
    // define setup parameters for OpenADC10
    // do not assign channels to scan
    #define PARAM5	SKIP_SCAN_ALL
    //Set Channel 0 ADC to AN5 to read potentiometer
    SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN5 ); 
    OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); 
    EnableADC10(); // Enable the ADC
  
    // ====================== SET UP PWM
    //Open Timer for Servo PWM. Use 16 prescaled cycles for appopriate frequency
    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_16, PWM_period);
    // set up compare1 for PWM output to servo
    OpenOC1(OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE , pwm_on_time, pwm_on_time); //
    // OC1 is PPS group 1, map to RPB4 (pin 11)
    PPSOutput(1, RPB4, OC1);
    SetDCOC1PWM(0); //Turn off PWM -- For some reason doesn't like init to 0.

    // =================  init the display
    tft_init_hw();
    tft_begin();
    tft_fillScreen(ILI9340_BLACK);
    //240x320 vertical display
    tft_setRotation(1); // Use tft_setRotation(1) for 320x240
    tft_setCursor(0, 0);
  
    // === init & Schedule the threads ====================
    PT_INIT(&pt_print);
    PT_INIT(&pt_control);
    
    while(1) {
      PT_SCHEDULE(protothread_print(&pt_print));
      PT_SCHEDULE(protothread_control(&pt_control));
    }
} // main
